@@ -1,5 +1,5 @@
 CQmodel <-
-function(p.est = NULL, show = NULL, p.type = NULL) {
+function(p.est = NULL, show = NULL, p.type = NULL, equation = NULL) {
 
 	############Helper functions############
 	
@@ -16,6 +16,7 @@ function(p.est = NULL, show = NULL, p.type = NULL) {
 
 
 	RMP <- function(table, parts) {
+		
 
 
 
@@ -34,6 +35,7 @@ function(p.est = NULL, show = NULL, p.type = NULL) {
 		parts.search[parts == 'step'] <- '(step|category)'
 
 		line.seps <- numeric()
+		
 
 		for (i in 1:length(parts)) {
 
@@ -43,6 +45,8 @@ function(p.est = NULL, show = NULL, p.type = NULL) {
 
 				temp.col.seps[1] <- gregexpr(parts.search[i],left.side.titles)[[1]][1] - 2
 				#print(temp.col.seps)
+				
+				
 
 				if (i > 1){
 
@@ -50,6 +54,8 @@ function(p.est = NULL, show = NULL, p.type = NULL) {
 					# print(temp.col.seps)
 	
 					if(parts[i] != 'step' & parts[i] != 'category'){
+						
+						
 	
 						first.line <- substr(table[7], 1, temp.col.seps[1]+2)
 						#print(first.line)
@@ -61,6 +67,8 @@ function(p.est = NULL, show = NULL, p.type = NULL) {
 						temp.col.seps[1] <- temp.col.seps[1] - temp.col.seps[2] - sum(line.seps)
 						
 						#print(temp.col.seps)
+						
+						
 	
 					}else{
 
@@ -90,6 +98,7 @@ function(p.est = NULL, show = NULL, p.type = NULL) {
 	
 
 		}
+		
 
 		if (length(line.seps)==0){
 
@@ -104,12 +113,22 @@ function(p.est = NULL, show = NULL, p.type = NULL) {
 		if (all(parts == 'step') & length(parts) == 1) {line.seps <- nchar(left.side.titles)}
 
 		#line.seps
+		#print(table)
 
 
 		out = grep("^ *(Parameter )?[0-9]", table, value = TRUE)
+		
+		#print(out)
 		out = gsub("[\\(\\)\\*,]", " ", out)
+		#print(out)
 		out = trim(out)
+		
+		if(nchar(out[1]) < sum(RMP.lengths)) {
+			RMP.lengths <- c(10,7)
+			RMP.titles <- c("est","error")
+		}
 		out <- split.right(out, sum(RMP.lengths))
+		
 
 		# out[,1] <- sub("(^[0-9]+)(\\s.+)", "\\1 \" \\2 \"", out[,1], perl=TRUE)
 		# if (length(titles)>2){ out[,1] <- sub("([0-9]\\s+\"$)", "\" \" \\1", out[,1], perl=TRUE)}
@@ -123,11 +142,13 @@ function(p.est = NULL, show = NULL, p.type = NULL) {
 			left.table[sapply(left.table, is.character)] <- sapply(left.table[sapply(left.table, is.character)],function (x) gsub("^\\s+|\\s+$", "", x))
 
 		}
+		
 		#left.table <- read.table(tempify(out[1]), col.names = titles, stringsAsFactors = FALSE)
 		# if (imported) {
 		# 	left.table[2] <- paste(left.table[[1]], left.table[[2]])
 		# 	left.table <- left.table[2]
 		# }
+		
 		right.table <- read.fwf(tempify(out[2]), RMP.lengths, col.names = RMP.titles, stringsAsFactors = FALSE)
 
 		cbind(left.table, right.table)
@@ -293,16 +314,16 @@ function(p.est = NULL, show = NULL, p.type = NULL) {
 
 		model <- breakup(shw, shw.starts, shw.titles)
 		model$imported <- imported
+		
+		#print(model)
 
-
-		#return(proc.time()-ptm)
 		
 		#######SOE####################
 		
 		#ptm <- proc.time()
 		date.pattern <- "(?:(Sun|Mon|Tue|Wed|Thu|Fri|Sat)\\s+)?(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\\s+(0[1-9]|[1-2]?[0-9]|3[01])\\s+(2[0-3]|[0-1][0-9]):([0-5][0-9])(?::(60|[0-5][0-9]))?\\s+(19[0-9]{2}|[2-9][0-9]{3})+$"
-		date.at <- grep(date.pattern, model$SOE)
-		title.date <- model$SOE[date.at]
+		date.at <- grep(date.pattern, model[[1]])
+		title.date <- model[[1]][date.at]
 		model$SOE <- safe.remove(model$SOE, date.at)
 		m <- regexpr(date.pattern, title.date)
 		model$run.details <- list()
@@ -324,59 +345,79 @@ function(p.est = NULL, show = NULL, p.type = NULL) {
 		two.colons.at <- grep(":.+:", colons)
 		other.lines <- append(other.lines, colons[two.colons.at])
 		colons <- safe.remove(colons, two.colons.at)
+		
+		if(!is.null(colons)) {
 
-		SOE <- split.by(colons, ":")
-
-		other.lines <- other.lines[-grep("===", other.lines)]
-		other.lines <- other.lines[other.lines != ""]
-		other.lines <- other.lines[other.lines != "SUMMARY OF THE ESTIMATION"]
-		other.lines <- trim(other.lines)
-
-		reason.at <- grep("Iterations terminated because", other.lines)
-		SOE$termination.reason <- other.lines[reason.at]
-		other.lines <- safe.remove(other.lines, reason.at)
-
-		deviance.line <- grep("Deviance Change=", other.lines, value = TRUE)
-		other.lines <- other.lines[-grep("Deviance Change=", other.lines)]
-		termination.criteria <- c(unlist(strsplit(SOE$criteria, ",")), deviance.line)
-		SOE <- c(SOE, split.by(termination.criteria, "="))
-		SOE$criteria <- NULL
-
-		model$run.details$format <- SOE$format
-		model$run.details$key <- SOE$key
-		model$run.details <- c(model$run.details, other.lines)
-		SOE <- numify(SOE)
-
-		model <- c(SOE["equation"], SOE["participants"], SOE["deviance"], SOE["parameters"], model)
-		class(SOE) <- "SOE"
+			SOE <- split.by(colons, ":")
+	
+			other.lines <- other.lines[-grep("===", other.lines)]
+			other.lines <- other.lines[other.lines != ""]
+			other.lines <- other.lines[other.lines != "SUMMARY OF THE ESTIMATION"]
+			other.lines <- trim(other.lines)
+	
+			reason.at <- grep("Iterations terminated because", other.lines)
+			SOE$termination.reason <- other.lines[reason.at]
+			other.lines <- safe.remove(other.lines, reason.at)
+	
+			deviance.line <- grep("Deviance Change=", other.lines, value = TRUE)
+			other.lines <- other.lines[-grep("Deviance Change=", other.lines)]
+			termination.criteria <- c(unlist(strsplit(SOE$criteria, ",")), deviance.line)
+			SOE <- c(SOE, split.by(termination.criteria, "="))
+			SOE$criteria <- NULL
+	
+			model$run.details$format <- SOE$format
+			model$run.details$key <- SOE$key
+			model$run.details <- c(model$run.details, other.lines)
+			SOE <- numify(SOE)
+	
+			model <- c(SOE["equation"], SOE["participants"], SOE["deviance"], SOE["parameters"], model)
+			class(SOE) <- "SOE"
+	
+	
+			model$SOE <- SOE
+		
+		}
+		
+		if(is.null(model$equation)) {
+			if(is.null(equation)) {
+				stop("Please specify the model equation.")
+			} else {
+				model$equation <- equation
+			}
+		}
+		
+		
 
 
-		model$SOE <- SOE
-
-
-		#return(proc.time()-ptm)
 		
 		##############RMP######################
 		
 
 
-		additive.parts = unlist(strsplit(SOE$equation, "[+|-]"))
+		additive.parts = unlist(strsplit(model$equation, "[+|-]"))
 		parts = strsplit(additive.parts, "\\*")
 		model$additive.parts <- additive.parts
 		model$parts <- parts
+		
+		
+		
 		if (imported) {
 			params <- RMP(model$RMP, "Parameters")
 			model$RMP <- list()
 			model$RMP$item <- params
 		} else {
-			RMP.tables <- breakup(model$RMP, grep("TERM ", model$RMP), additive.parts)
+			RMP.tables <- breakup(model$RMP, grep("  VARIABLES", model$RMP) - 2, additive.parts)
 			model$RMP = mapply(RMP, RMP.tables, parts, SIMPLIFY = FALSE)
 			model$run.details$names <- mapply(get.names, parts[parts == additive.parts], model$RMP[parts == additive.parts])
 		}
 		
+		
+		
+		#print(model$RMP)
+		
 
 		##########PMP###########
-		
+		if(!is.null(model$PMP)) {
 		PMP.starts <- grep("^====+", model$PMP)
 		PMP.heads <- grep(date.string, model$PMP)
 		PMP.starts <- PMP.starts[!(PMP.starts + 1) %in% PMP.heads]
@@ -385,13 +426,27 @@ function(p.est = NULL, show = NULL, p.type = NULL) {
 		variance.line = grep("Variance", PMP$cov.cor, value = TRUE)
 		m <- gregexpr("[0-9]+\\.[0-9]+(?![0-9])(?![\\s]*\\))", variance.line, perl = TRUE)
 		PMP$variances <- as.numeric(unlist(regmatches(variance.line, m)))
-		m <- gregexpr("[0-9\\.]+(?=[\\s]*\\))", variance.line, perl = TRUE)
+		m <- gregexpr("[0-9\\.\\-#IO]+(?=[\\s]*\\))", variance.line, perl = TRUE)
 		errors <- as.numeric(unlist(regmatches(variance.line, m)))
 		PMP$nDim = length(PMP$variances)
+		length(errors) <- length(PMP$variances)
 		PMP$dimensions = "Main dimension"
 		if (PMP$nDim > 1) {
-			PMP$cov.cor <- read.fwf(tempify(PMP$cov.cor[8:(8 + PMP$nDim - 1)]), widths = c(25, rep(9, PMP$nDim)), row.names = 1, 
+
+			if (CQV == 2){
+				
+				PMP$cov.cor <- read.fwf(tempify(PMP$cov.cor[8:(8 + PMP$nDim - 1)]), widths = c(25, rep(9, PMP$nDim)), row.names = 1, 
 				strip.white = TRUE, stringsAsFactors = FALSE)
+				
+			}
+			
+			if (CQV == 3){
+				
+				PMP$cov.cor <- read.fwf(tempify(PMP$cov.cor[8:(8 + PMP$nDim - 1)]), widths = c(25, rep(18, PMP$nDim)), row.names = 1, 
+				strip.white = TRUE, stringsAsFactors = FALSE)
+				
+			}
+
 			PMP$dimensions <- row.names(PMP$cov.cor)
 			names(PMP$cov.cor) <- PMP$dimensions
 			PMP$cor.matrix <- PMP$cov.cor
@@ -426,13 +481,19 @@ function(p.est = NULL, show = NULL, p.type = NULL) {
 
 		model$PMP <- NULL
 		model <- c(model, PMP)
+		
+		}
+		
 
 
 		########GIN#########
 		
 		do.GIN <- function(GIN) {
-			GIN <- GIN[7:(length(GIN))]
+			
+			GIN <- GIN[4:(length(GIN))]
 			GIN <- GIN[-grep("===",GIN)]
+			GIN <- GIN[-grep("GIN Number", GIN)]
+			GIN <- GIN[-grep("---",GIN)]
 
 			GIN <- gsub("\\s*\t\\s*", "\t", GIN)
 			GIN <- gsub(" +","_",GIN)
@@ -443,13 +504,16 @@ function(p.est = NULL, show = NULL, p.type = NULL) {
 			return(make.GIN(GIN))
 		}
 		
+		
 		if (!is.null(model$GIN)) {
 			model$GIN <- do.GIN(model$GIN)
 		}
 		
+		
 		if(!is.null(model$GIN.deltas)) {
 			model$GIN.deltas <- do.GIN(model$GIN.deltas)
 		}
+		
 		#return(proc.time()-ptm)
 		class(model) <- "CQmodel"
 
@@ -472,7 +536,7 @@ function(p.est = NULL, show = NULL, p.type = NULL) {
 			colperdim = 3
 		else colperdim = 4
 
-		model$nDim = floor(length(p.est)/colperdim)
+		model$nDim = floor((length(p.est) - 1)/colperdim)
 
 		if (is.null(model$dimensions) || length(model$dimensions != model$nDim)) 
 			model$dimensions <- paste("d", c(1:model$nDim), sep = "")
@@ -486,9 +550,17 @@ function(p.est = NULL, show = NULL, p.type = NULL) {
 
 		if (length(p.est)%%colperdim == 1) {
 			names(p.est) <- c("casenum", pp_lab_t)
+		} else if (p.type != "WLE") {
+			names(p.est) <- c("casenum", "pid", pp_lab_t)
+		} else if (length(p.est)%%colperdim == 3) {
+			names(p.est) <- c("casenum","pid", pp_lab_t, "case.fit")
+		} else if (any(p.est[4] < p.est[3])) {
+			names(p.est) <- c("casenum", pp_lab_t, "case.fit")
 		} else {
 			names(p.est) <- c("casenum", "pid", pp_lab_t)
 		}
+		
+		
 
 		model$p.est <- p.est
 		model$p.est.type <- p.type
